#include <p33FJ128GP202.h>
#include <stdlib.h>
#include "c_what.h"
#include "lj_yeah2.h"
#include "lj_yeah3.h"
#include "lj_yeah4.h"
#include "lj_okay.h"
#include "lj_what.h"
#include "c_yeah1.h"
#include "c_yeah2.h"
#include "c_okay.h"
#include "c_yeah4.h"

//number of wavetables
const int NSOUND = 10;
const int PWM_RANGE = 0x0FFF; //2^(12-OVERSAMPLE_BITS)
int cSound = 0; //current sound selected
int cIdx = -1; //current sample index
unsigned long iTime; //idle time
//array of pointers to wavetables
__prog__ unsigned char* wPtr[] = {c_what, lj_yeah2, lj_yeah3, lj_yeah4, lj_okay, lj_what, c_yeah1, c_yeah2, c_okay, c_yeah4,};
//array of wavetable lengths
int wSizes[] = {sizeof(c_what), sizeof(lj_yeah2), sizeof(lj_yeah3), sizeof(lj_yeah4), sizeof(lj_okay), sizeof(lj_what), sizeof(c_yeah1), sizeof(c_yeah2), sizeof(c_okay), sizeof(c_yeah4)};


//FIR filter for interpolat
void __attribute__((__interrupt__)) _T2Interrupt( void )
{
	//Interrupt for PWM to signal it needs more data (Timer2 overflowed)
	if (cIdx >= 0)
	{
	
		//Load data into PWM Duty Cycle Register (convert 8-bit to 12-bit data)
		OC1RS = wPtr[cSound][cIdx] * 16;
		cIdx ++;
		if (cIdx > wSizes[cSound])
		{
			//map PWM to default (something else, simpler than stopping)
			RPOR7 = 0x0000;
			//set PWM output to 0 for NPN output driver (set to 1 for PNP)
			PORTBbits.RB14 = 0;
			cIdx = -1;	//setting it to negative value means not playing
		}
		iTime = 0;
	}		
	IFS0bits.T2IF = 0; // Clear Timer 2 interrupt flag
}

void __attribute__((__interrupt__)) _CNInterrupt( void )
{
	//Interrupt for change notification
	if (PORTAbits.RA4)
	{
		//play a new sound if previously stopped or past 3/8th of a second
		//else randomly pick a sound
		if (cIdx < 0 || cIdx > 3000)
			cSound = rand()%NSOUND;
		cIdx = 0;
		//map PWM to output (simpler than starting PWM)
		RPOR7 = 0b0000000000010010;
		iTime = 0;
	}
	
	IFS1bits.CNIF = 0;
	
}
void setupHS()
{
	//N2 = 2
	CLKDIVbits.PLLPOST = 0b00;
	//N1 = 2
	CLKDIVbits.PLLPRE = 0b00000;
	CLKDIVbits.FRCDIV = 0b000;
	PLLFBD = 36-2;
	//Total PLL factor = (36/(2*2)) = 9
	//Clock speed = 7.37 * 9 = 66.33 Mhz;
}

void setupCN()
{
	//Enable change notification pin 0(CN0), this lets me put the MCU to sleep until
	//it's needed to play something and thus save power
	TRISA = 0xFFFF;
	//Enable the internal weak pullup (saves me a resistor)
	CNPU1bits.CN0PUE = 1;
	//Enable interrupt for CN0
	CNEN1bits.CN0IE = 1;
	//Enable overall CN interrupt
	IFS1bits.CNIF = 0;
	IEC1bits.CNIE = 1;
}

void setupPWM()
{
	//Set PORTB to outputs
	TRISB = 0x0000;
	PORTB = 0x0000;
	
	//Initialize Output Compare Module
	OC1CONbits.OCM = 0b000; // Disable Output Compare Module
	OC1R = 100; // Write the duty cycle for the first PWM pulse
	OC1RS = 200; // Write the duty cycle for the second PWM pulse
	OC1CONbits.OCTSEL = 0; // Select Timer 2 as output compare time base
	OC1R = 100; // Load the Compare Register Value
	OC1CONbits.OCM = 0b110; // Select the Output Compare mode
	
	//Initialize and enable Timer2
	//These settings give me approximately 12-bit PWM at 8KHz
	T2CONbits.TON = 0; // Disable Timer
	T2CONbits.TCS = 0; // Select internal instruction cycle clock
	T2CONbits.TGATE = 0; // Disable Gated Timer mode
	T2CONbits.TCKPS = 0b00; // Select 1:1 Prescaler
	TMR2 = 0x00; // Clear timer register
	PR2 = PWM_RANGE; // Load the period value
	IPC1bits.T2IP = 0x01; // Set Timer 2 Interrupt Priority Level
	IFS0bits.T2IF = 0; // Clear Timer 2 Interrupt Flag
	IEC0bits.T2IE = 1; // Enable Timer 2 interrupt
	T2CONbits.TON = 1; // Start Timer
	//Example code for Timer 2 ISR
}

int main ()
{
	setupHS();
	setupPWM();
	setupCN();
	while (1)
	{	
		//wait for a bit and then enter sleep
		if (iTime > 0x000FFFFF)
		{
			PORTBbits.RB10 = 0;
			iTime = 0;
			Sleep();
		}
		else
		{
			PORTBbits.RB10 = 1;
			iTime ++;
		}		
	}
	return 0;
}
